import shutil
from collections.abc import Generator
from pathlib import Path
from typing import Any

import pytest

from videoxt.api import (
    extract,
    extract_audio,
    extract_clip,
    extract_frames,
    extract_gif,
)
from videoxt.exceptions import InvalidExtractionMethod
from videoxt.result import Result


@pytest.fixture(scope="session")
def extract_frames_result(fixture_tmp_video_filepath) -> Generator[Result, None, None]:
    """Extract frames from a video and yield a Result object."""
    result = extract_frames(fixture_tmp_video_filepath)
    yield result
    try:
        shutil.rmtree(Path(result.destpath))
    except FileNotFoundError:
        pass


@pytest.fixture(scope="session")
def generic_extract_frames_result(
    fixture_tmp_video_filepath,
) -> Generator[Result, None, None]:
    """Extract frames from a video using generic extract() and yield a Result object."""
    result = extract("frames", fixture_tmp_video_filepath, overwrite=True)
    yield result
    try:
        shutil.rmtree(Path(result.destpath))
    except FileNotFoundError:
        pass


def test_extract_frames_valid_default_request_success_is_true(
    extract_frames_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    """Test that the default request for extracting frames is successful."""
    assert extract_frames_result.success is True
    assert extract_frames_result.method == "frames"
    assert extract_frames_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent / "tmp.video.mp4_frames"
    )
    assert extract_frames_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_dir()


def test_generic_extract_frames_valid_default_request_success_is_true(
    generic_extract_frames_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    """Test that the default request for extracting frames is successful."""
    assert generic_extract_frames_result.success is True
    assert generic_extract_frames_result.method == "frames"
    assert generic_extract_frames_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent / "tmp.video.mp4_frames"
    )
    assert generic_extract_frames_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_dir()


@pytest.fixture(scope="session")
def extract_audio_result(fixture_tmp_video_filepath) -> Generator[Result, None, None]:
    """Extract audio from a video and yield a Result object."""
    result = extract_audio(fixture_tmp_video_filepath)
    yield result
    try:
        Path(result.destpath).unlink()
    except FileNotFoundError:
        pass


@pytest.fixture(scope="session")
def generic_extract_audio_result(
    fixture_tmp_video_filepath,
) -> Generator[Result, None, None]:
    """Extract audio from a video using generic extract() and yield a Result object."""
    result = extract("audio", fixture_tmp_video_filepath, overwrite=True)
    yield result
    try:
        Path(result.destpath).unlink()
    except FileNotFoundError:
        pass


def test_extract_audio_valid_default_request_success_is_true(
    extract_audio_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    assert extract_audio_result.success is True
    assert extract_audio_result.method == "audio"
    assert extract_audio_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent / "tmp.video.mp3"
    )
    assert extract_audio_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_file()


def test_generic_extract_audio_valid_default_request_success_is_true(
    generic_extract_audio_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    assert generic_extract_audio_result.success is True
    assert generic_extract_audio_result.method == "audio"
    assert generic_extract_audio_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent / "tmp.video.mp3"
    )
    assert generic_extract_audio_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_file()


@pytest.fixture(scope="session")
def extract_clip_result(fixture_tmp_video_filepath) -> Generator[Result, None, None]:
    """Extract a clip from a video and yield a Result object."""
    result = extract_clip(fixture_tmp_video_filepath, filename="tmp.test.extract.clip")
    yield result
    try:
        Path(result.destpath).unlink()
    except FileNotFoundError:
        pass


@pytest.fixture(scope="session")
def generic_extract_clip_result(
    fixture_tmp_video_filepath,
) -> Generator[Result, None, None]:
    """Extract a clip from a video using generic extract() and yield a Result object."""
    result = extract(
        "clip",
        fixture_tmp_video_filepath,
        filename="tmp.test.extract.clip",
        overwrite=True,
    )
    yield result
    try:
        Path(result.destpath).unlink()
    except FileNotFoundError:
        pass


def test_extract_clip_valid_default_request_success_is_true(
    extract_clip_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    assert extract_clip_result.success is True
    assert extract_clip_result.method == "clip"
    assert extract_clip_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent
        / "tmp.test.extract.clip.mp4"
    )
    assert extract_clip_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_file()


def test_generic_extract_clip_valid_default_request_success_is_true(
    generic_extract_clip_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    assert generic_extract_clip_result.success is True
    assert generic_extract_clip_result.method == "clip"
    assert generic_extract_clip_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent
        / "tmp.test.extract.clip.mp4"
    )
    assert generic_extract_clip_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_file()


@pytest.fixture(scope="session")
def extract_gif_result(fixture_tmp_video_filepath) -> Generator[Result, None, None]:
    """Extract a gif from a video and yield a Result object."""
    result = extract_gif(fixture_tmp_video_filepath)
    yield result
    try:
        Path(result.destpath).unlink()
    except FileNotFoundError:
        pass


@pytest.fixture(scope="session")
def generic_extract_gif_result(
    fixture_tmp_video_filepath,
) -> Generator[Result, None, None]:
    """Extract a gif from a video using generic extract() and yield a Result object."""
    result = extract("gif", fixture_tmp_video_filepath, overwrite=True)
    yield result
    try:
        Path(result.destpath).unlink()
    except FileNotFoundError:
        pass


def test_extract_gif_valid_default_request_success_is_true(
    extract_gif_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    assert extract_gif_result.success is True
    assert extract_gif_result.method == "gif"
    assert extract_gif_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent / "tmp.video.gif"
    )
    assert extract_gif_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_file()


def test_generic_extract_gif_valid_default_request_success_is_true(
    generic_extract_gif_result: Generator[Result, None, None],
    fixture_tmp_video_properties: dict[str, Any],
):
    assert generic_extract_gif_result.success is True
    assert generic_extract_gif_result.method == "gif"
    assert generic_extract_gif_result.message == "Extraction successful."
    expected_destpath = (
        fixture_tmp_video_properties["video_file_path"].parent / "tmp.video.gif"
    )
    assert generic_extract_gif_result.destpath == expected_destpath
    expected_destpath = Path(expected_destpath)
    assert expected_destpath.exists()
    assert expected_destpath.is_file()


def test_extract_with_unsupported_extraction_method_raises_invalid_extraction_method_error(
    fixture_tmp_video_filepath,
):
    """Test that an invalid extraction method raises an InvalidExtractionMethodError."""
    with pytest.raises(InvalidExtractionMethod) as excinfo:
        extract("unsupported", fixture_tmp_video_filepath)
    assert "Invalid extraction method" in str(excinfo.value)



import argparse
import shutil

import pytest

from videoxt.cli import execute_extraction, main, split_cli_args


def test_split_cli_args():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest="subparser_name")
    subparser = subparsers.add_parser("audio")
    subparser.add_argument("filepath")
    subparser.add_argument("--option1")
    subparser.add_argument("--option2")

    args = parser.parse_args(["audio", "input_file.mp4", "--option1", "value1"])

    extraction_method, filepath, options = split_cli_args(args)

    assert extraction_method == "audio"
    assert filepath == "input_file.mp4"
    assert options == {"option1": "value1", "option2": None}


def test_execute_extraction_returns_exit_code_0_for_valid_video_file(
    fixture_tmp_video_filepath,
):
    destdir = fixture_tmp_video_filepath.parent
    filename = "tmp.execute.extraction.audio"
    destpath = destdir / f"{filename}.mp3"
    exit_code = execute_extraction(
        method="audio",  # quickest extraction method
        filepath=str(fixture_tmp_video_filepath),
        **{"destdir": destdir, "filename": filename},
    )
    assert exit_code == 0
    assert destpath.exists()

    try:
        destpath.unlink()
    except FileNotFoundError:
        pass


def test_execute_extraction_returns_exit_code_1_for_invalid_video_file(
    fixture_tmp_video_filepath_zero_seconds,
):
    exit_code = execute_extraction(
        method="audio",
        filepath=str(fixture_tmp_video_filepath_zero_seconds),
        **{},
    )
    assert exit_code == 1


@pytest.mark.parametrize(
    "method, suffix", [("audio", "mp3"), ("clip", "mp4"), ("gif", "gif")]
)
def test_main_successful_extraction_returns_exit_code_0(
    fixture_tmp_video_filepath, method, suffix
):
    destdir = fixture_tmp_video_filepath.parent
    filename = "tmp.main.successful.extraction"
    destpath = destdir / f"{filename}.{suffix}"
    exit_code = main(
        [
            method,
            str(fixture_tmp_video_filepath),
            "--filename",
            filename,
            "--destdir",
            str(destdir),
        ]
    )
    assert exit_code == 0
    assert destpath.exists()

    try:
        destpath.unlink()
    except FileNotFoundError:
        pass


def test_main_successful_frames_extraction_returns_exit_code_0(
    fixture_tmp_video_filepath,
):
    destdir = fixture_tmp_video_filepath.parent / "frames"
    destdir.mkdir(parents=True, exist_ok=True)
    exit_code = main(
        [
            "frames",
            str(fixture_tmp_video_filepath),
            "--destdir",
            str(destdir),
        ]
    )
    assert exit_code == 0
    assert destdir.exists()

    try:
        shutil.rmtree(destdir)
    except FileNotFoundError:
        pass


def test_main_with_invalid_positional_arg_raises_argparse_argument_error_and_returns_exit_code_1(
    fixture_tmp_video_filepath,
):
    with pytest.raises(SystemExit):
        exit_code = main(["invalid_positional_arg", str(fixture_tmp_video_filepath)])
        assert exit_code == 0


@pytest.mark.parametrize("method", ["audio", "clip", "frames", "gif"])
def test_main_with_invalid_video_file_returns_exit_code_1(
    fixture_tmp_video_filepath_zero_seconds, method
):
    exit_code = main([method, str(fixture_tmp_video_filepath_zero_seconds)])
    assert exit_code == 1


@pytest.mark.parametrize("method", ["audio", "clip", "frames", "gif"])
def test_main_with_unrecognized_option_returns_exit_code_1(
    fixture_tmp_video_filepath, method
):
    with pytest.raises(SystemExit):
        exit_code = main([method, str(fixture_tmp_video_filepath), "--invalid_option"])
        assert exit_code == 1


@pytest.mark.parametrize("method", ["audio", "clip", "frames", "gif"])
def test_main_argument_type_error_returns_exit_code_1(
    fixture_tmp_video_filepath, method
):
    exit_code = main([method, str(fixture_tmp_video_filepath), "--start-time", "-1"])
    assert exit_code == 1


@pytest.mark.parametrize("method", ["audio", "clip", "frames", "gif"])
def test_main_videoxt_error_returns_exit_code_1(fixture_tmp_video_filepath, method):
    exit_code = main(
        [
            method,
            str(fixture_tmp_video_filepath),
            "--start-time",
            "2",
            "--stop-time",
            "1",
        ]
    )
    assert exit_code == 1



"""Containers for extraction results."""
from dataclasses import dataclass
from pathlib import Path

from videoxt.utils import ToJsonMixin


@dataclass
class Result(ToJsonMixin):
    """
    A container for the result of an extraction.

    Attributes:
    -----
        `success` (bool | None):
            Whether the extraction was successful.
        `method` (str | None):
            The name of the method that was called.
        `message` (str | None):
            A message describing the result of the extraction.
        `destpath` (Path | None):
            The path to the destination file or directory.
        `elapsed_time` (float | None):
            The number of seconds it took to complete the extraction.

    Methods:
    -----
        - `json()` -> `str`: Return a JSON string representation of the request.
    """

    success: bool | None = None
    method: str | None = None
    message: str | None = None
    destpath: Path | None = None
    elapsed_time: float | None = None
